/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 342 (Exercise B.1.5)
**
*/

new;
library rpb,pgraph;

cls;
__output = 0;

let mu =    0.15 0.10 0.08 0.06;
let sigma = 0.15 0.10 0.07 0.05;
let rho = 1.00 
          0.50 1.00
          0.20 0.20 1.00
          0.00 0.00 0.00 1.00;
rho = xpnd(rho);
cov = rho .* sigma .* sigma';
call invpd(cov);

A = ones(1,4); B = 1; C = 0; D = 0; Bounds = (0~1) .* ones(4,1);

print;
print "=====================  Question 1  =======================================";

Results = zeros(8,3);

gamma_ = seqa(0,0.0001,10001);
{x,er,risk,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,C,D,Bounds,gamma_,0);

rf = 0.02;
sh = (er - rf)./risk;  
indx = maxindc(sh);
Results[.,1] = rf|x[.,indx]|er[indx]|risk[indx]|sh[indx];

rf = 0.03;
sh = (er - rf)./risk;  
indx = maxindc(sh);
Results[.,2] = rf|x[.,indx]|er[indx]|risk[indx]|sh[indx];

rf = 0.04;
sh = (er - rf)./risk;  
indx = maxindc(sh);
Results[.,3] = rf|x[.,indx]|er[indx]|risk[indx]|sh[indx];

print ftosa(100*Results,"%lf",3,2);

print;
print "=====================  Question 1  =======================================";

gamma_star = 0|0.25;
cov_star = (cov ~ zeros(4,1)) | zeros(1,5);
A_star = ones(1,5); B_star = 1; C_star = 0; D_star = 0; Bounds_star = (-10~10) .* ones(5,1);

rf = 0.02;
mu_star = mu | rf;
{x,er,risk,gamma_x,retcode} = Compute_MVO_Portfolio(mu_star,cov_star,A_star,B_star,C_star,D_star,Bounds_star,gamma_star,0);
x1 = x[.,1]; x2 = x[.,2];
alpha = x1[5]/(x1[5] - x2[5]);
x = (1-alpha) * x1 + alpha * x2;
er = x'mu_star; risk = sqrt(x'*cov_star*x); sh = (er - rf)/risk;  
Results[.,1] = rf|trimr(x,0,1)|er|risk|sh;

rf = 0.03;
mu_star = mu | rf;
{x,er,risk,gamma_x,retcode} = Compute_MVO_Portfolio(mu_star,cov_star,A_star,B_star,C_star,D_star,Bounds_star,gamma_star,0);
x1 = x[.,1]; x2 = x[.,2];
alpha = x1[5]/(x1[5] - x2[5]);
x = (1-alpha) * x1 + alpha * x2;
er = x'mu_star; risk = sqrt(x'*cov_star*x); sh = (er - rf)/risk;  
Results[.,2] = rf|trimr(x,0,1)|er|risk|sh;

x_opt_sh = trimr(x,0,1);

rf = 0.04;
mu_star = mu | rf;
{x,er,risk,gamma_x,retcode} = Compute_MVO_Portfolio(mu_star,cov_star,A_star,B_star,C_star,D_star,Bounds_star,gamma_star,0);
x1 = x[.,1]; x2 = x[.,2];
alpha = x1[5]/(x1[5] - x2[5]);
x = (1-alpha) * x1 + alpha * x2;
er = x'mu_star; risk = sqrt(x'*cov_star*x); sh = (er - rf)/risk;  
Results[.,3] = rf|trimr(x,0,1)|er|risk|sh;

print ftosa(100*Results,"%lf",3,2);

print;
print "=====================  Question 2.a & 2.b  =======================================";

Results = zeros(10,6);

rf = 0.03;
let x0 = 0.60 0.30 0.10 0.00;
er = x0'mu;
risk = sqrt(x0'cov*x0);
Sh = (er-rf)/risk;

Results[.,1] = x0|er|risk|sh|0|0|0;

{x,alpha,te,gamma_x,retcode} = Compute_TE_Minimum(x0,mu,cov,A,B,C,D,Bounds);
IR = alpha/te;
er = x'mu; risk = sqrt(x'cov*x); Sh = (er-rf)/risk;
Results[.,2] = x|er|risk|sh|alpha|te|IR;

{x,alpha,te,gamma_x,retcode} = Compute_TE_Maximum(x0,mu,cov,A,B,C,D,Bounds);
IR = alpha/te;
er = x'mu; risk = sqrt(x'cov*x); Sh = (er-rf)/risk;
Results[.,3] = x|er|risk|sh|alpha|te|IR;

{x,alpha,te,gamma_x,retcode} = Compute_TE_Portfolio(x0,mu,cov,A,B,C,D,Bounds,0.03,2);
IR = alpha/te;
er = x'mu; risk = sqrt(x'cov*x); Sh = (er-rf)/risk;
Results[.,4] = x|er|risk|sh|alpha|te|IR;

gamma_ = seqa(0,0.001,1001);
{x,alpha,te,gamma_x,retcode} = Compute_TE_Portfolio(x0,mu,cov,A,B,C,D,Bounds,gamma_,0);
x_all = x;
IR = alpha./te;
indx = maxindc(IR);
x = x[.,indx]; er = x'mu; risk = sqrt(x'cov*x); Sh = (er-rf)/risk;
Results[.,5] = x|er|risk|sh|alpha[indx]|te[indx]|IR[indx];

IR[1] = error(0);
indx = sumc(x_all[3,.]' .>= 1e-5);

x1 = x_all[.,indx/2];
te_max = te[indx];
IR_max = IR[indx/2];

graphset;
  window3d;

    fonts("simplex simgrma");
    _pdate = ""; _pnum = 2; _pframe = 0;
    _pltype = 6; _plwidth = 10; _pcolor = 1;
    _paxht = 0.225; _pnumht = 0.225; _ptitlht = 0.30;

  setwind(1);
    ylabel("\216\202m\201(x | b) (in %)");
    xlabel("\216\202s\201(x | b) (in %)");
    xtics(0,5,1,2);
    ytics(0,2,0.5,2);
    _pline = 1~3~0~0~5~5*IR[indx]~1~0~5 |
             1~3~100*te[indx]~0~100*te[indx]~2~1~4~10;
    xy(100*te,100*alpha);

  setwind(2);
    xlabel("\216\202s\201(x | b) (in %)");
    ylabel("\216IR(x | b) (in %)");
    xtics(0,5,1,2);
    ytics(43,47,1,2);
    _pline = 1~3~100*te[indx]~43~100*te[indx]~47~1~4~10;
    xy(100*te,100*IR);

  setwind(3);
    xlabel("\216\202s\201(x | b) (in %)");
    ylabel("\216x]3[ (in %)");
    xtics(0,5,1,2);
    ytics(0,10,2,2);
    _pline = 1~3~100*te[indx]~0~100*te[indx]~10~1~4~10;
    xy(100*te,100*x_all[3,.]');

    graphprt("-c=2 -cf=app2-1-5-2.eps");

  endwind;

te = seqa(0.000,0.0001,50);
{x,alpha,te,gamma_x,retcode} = Compute_TE_Portfolio(x0,mu,cov,A,B,C,D,Bounds,te,2);
IR = alpha./te;
indx = maxindc(IR);
x = x[.,indx]; er = x'mu; risk = sqrt(x'cov*x); Sh = (er-rf)/risk;
Results[.,6] = x|er|risk|sh|alpha[indx]|te[indx]|IR[indx];

print ftosa(100*Results,"%lf",3,2);

print ftosa(100*x1,"%lf",3,4);
print;
print ftos(100*IR_max,"IR(max) = %lf",3,4);
print ftos(100*te_max,"te(max) = %lf",3,4);

print;
print "=====================  Question 2.c  =======================================";

Bounds = ones(4,1) .* (0.10~0.50);
{x_min,alpha_min,te_min,gamma_mi,retcode} = Compute_TE_Minimum(x0,mu,cov,A,B,C,D,Bounds);
ir_min = alpha_min/te_min;

{x_max,alpha_max,te_max,gamma_max,retcode} = Compute_TE_Maximum(x0,mu,cov,A,B,C,D,Bounds);
ir_max = alpha_max/te_max;

print ftosa(100*(x_min~x_max),"%lf",3,2);
print ftosa(100*(te_min~te_max),"te = %lf",3,2);
print ftosa(100*(ir_min~ir_max),"ir = %lf",3,2);

gamma_ = seqa(0,0.001,1001);
{x,alpha,te,gamma_x,retcode} = Compute_TE_Portfolio(x0,mu,cov,A,B,C,D,Bounds,gamma_,0);
IR = alpha./te;
indx = maxindc(IR);
x = x[.,indx]; er = x'mu; risk = sqrt(x'cov*x); Sh = (er-rf)/risk;
Results = x|er|risk|sh|alpha[indx]|te[indx]|IR[indx];

print;
print "=====================  Question 2.d  =======================================";

x = x_opt_sh;
surperf = (x-x0)'mu; 
te = sqrt((x-x0)'cov*(x-x0)); 
ir = surperf/te;
print "x = ";
print ftosa(100*x',"%lf",3,2);
print ftos(ir,"IR = %lf",3,2);
