/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 346 (Exercise B.1.9)
**
*/

new;
library rpb,pgraph;

cls;

let er_mu    =    0.10 -0.05 0.15;
let sigma_mu =    0.04  0.02 0.10; 
let sigma_mu2 =   0.01  0.05 0.15; 
let sigma    =    0.20  0.20 0.15;
n = rows(sigma);
let rho =   1.00 
            0.50   1.00
            0.20   0.60   1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';

r = 0.0;
SR = 0.5;
x0 = ones(n,1)/n;

A = ones(1,n);
B = 1;
Bounds = (-1~1) .* ones(n,1);

nTau = 501;
te_target = seqa(0,0.05/(nTau-1),nTau);
mu_te = zeros(nTau,3);
sigma_te = zeros(nTau,3);
{x,mu_te[.,1],sigma_te[.,1],gamma_markowitz,retcode} = Compute_TE_Portfolio(x0,er_mu,cov,A,B,0,0,Bounds,te_target,2);

let te_target = 0.01 0.02 0.03 0.04 0.05;

print;
print "=====================  Question 1  =======================================";

A = ones(1,n);
B = 1;
Bounds = (-1~1) .* ones(n,1);

{x1,mu_te1,sigma_te1,gamma_markowitz,retcode} = Compute_TE_Portfolio(x0,er_mu,cov,A,B,0,0,Bounds,te_target,2);

Results = 100*x1|100*mu_te1'|100*sigma_te1';
print ftosa(Results,"%lf",3,2);

print;
print "=====================  Question 2.a  =======================================";

sigma_x0 = sqrt(x0'cov*x0);
phi = SR/sigma_x0;
mu_tilde = r + SR .* (cov * x0)/sigma_x0;

print ftosa(phi,"phi = %lf",3,4);
print ftosa(100*mu_tilde,"%lf",3,2);

print;
print "=====================  Question 2.b  =======================================";

P = eye(n);
Q = er_mu;
Omega = diagrv(eye(n),sigma_mu^2);
Omega2 = diagrv(eye(n),sigma_mu2^2);

print;
print "=====================  Question 2.c  =======================================";
tau = 0.01;
Gamma_ = tau * cov;
mu_bar = mu_tilde + Gamma_*P'invpd(Omega + P*Gamma_*P')*(Q-P*mu_tilde);
mu_bar2 = mu_tilde + Gamma_*P'invpd(Omega2 + P*Gamma_*P')*(Q-P*mu_tilde);

print ftosa(100*mu_bar,"%lf",3,2);

print ftosa((mu_bar~mu_bar2)./er_mu,"%lf",3,2);


print;
print "=====================  Question 2.d  =======================================";

gamma_markowitz = 1/phi;
{x2,er2,sigma2,gamma2,retcode} = Compute_MVO_Portfolio(mu_bar,cov,1,0,0,0,Bounds,gamma_markowitz,0);
mu_te2 = (x2-x0)'er_mu;
sigma_te2 = sqrt((x2-x0)'cov*(x2-x0));

Results = 100*x2|100*mu_te2'|100*sigma_te2';
print ftosa(Results,"%lf",3,2);

print;
print "=====================  Question 3.a  =======================================";

tau = 0.000;
Gamma_ = tau * cov;
mu_bar = mu_tilde + Gamma_*P'invpd(Omega + P*Gamma_*P')*(Q-P*mu_tilde);
{x3,er3,sigma3,gamma3,retcode} = Compute_MVO_Portfolio(mu_bar,cov,1,0,0,0,Bounds,gamma_markowitz,0);
mu_te3 = (x3-x0)'er_mu;
sigma_te3 = sqrt((x3-x0)'cov*(x3-x0));

Results = 100*x3|100*mu_te3'|100*sigma_te3';
print ftosa(Results,"%lf",3,2);

print;
print "=====================  Question 3.b  =======================================";

te_target = 0.03;

proc (1) = f_tau(tau);
  local Gamma_,x,mu_x,sigma_x,sigma_te,retcode;
  Gamma_ = tau * cov;
  {x,mu_x,sigma_x,sigma_te,retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);
  retp(sigma_te - te_target);
endp;

tau_star = Bisection(&f_tau,0,1);
Gamma_ = tau_star * cov;
{x4,er4,sigma4,sigma_te4,retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);
mu_te4 = er_mu'(x4-x0);
print ftosa(tau_star,"tau* = %lf",3,5);

Results = 100*x4|100*mu_te4'|100*sigma_te4';
print ftosa(Results,"%lf",3,2);

print;
print "=====================  Question 3.c  =======================================";

tau = seqa(0,0.01/(nTau-1),nTau);
nTau = rows(tau);
x_star = zeros(n,nTau);

i = 1;
do until i > nTau;
  Gamma_ = tau[i] * cov;
  {x_star[.,i],mu_x,sigma_x,sigma_te[i,2],retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);
  mu_te[i,2] = er_mu'(x_star[.,i]-x0);
  {x,mu_x,sigma_x,sigma_te[i,3],retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega2,Bounds);
  mu_te[i,3] = er_mu'(x-x0);
  i = i + 1;
endo;

indx = minindc(abs(sigma_te[.,2]-te_target));
x5 = x_star[.,indx];
mu_te5 = mu_te[indx,2];
sigma_te5 = sigma_te[indx,2];

Results = Results ~(100*x5|100*mu_te5'|100*sigma_te5');

print ftosa(Results,"%lf",3,2);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6|1|3; _pcolor = 1|4|2;
  ylabel("\216\202m\201(x | b) (in %)");
  xlabel("\216\202s\201(x | b) (in %)");
  xtics(0,4,1,2);
  ytics(-1,5,1,2);
  _plegstr = "TE model\000BL model\000BL model (#2)";
  _plegctl = {2 5 2 5};
  graphprt("-c=2 -cf=app2-1-9-1.eps");
  xy(100*sigma_te,100*mu_te);
