/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 10 (Figure 1.3 & Table 1.4)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(Sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';
A = ones(1,n); B = 1;

Bounds = (-10~10) .* ones(n,1);
gamma_ = seqa(-2,1/100,600);
{x0,mu0,sigma0,gamma0,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  

Bounds = (0~1) .* ones(n,1);
gamma_ = -5|-4|-3|seqa(-2,1/100,595)|4|50;
{x1,mu1,sigma1,gamma1,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  

Bounds = (0~0.4) .* ones(n,1);
gamma_ = -8|-5|-4|-3|seqa(-2,1/100,594)|5|50;
{x2,mu2,sigma2,gamma2,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2|4;
  xtics(10,25,2.5,2);
  ytics(3.5,8.5,1,0);
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected return (in %)");
  _plegstr = "No constraint\000x]i[ \202\62 \2010%\0000% \202\60 \201x]i[ \202\60 \20140%";
  _plegctl = {2 5 6.0 2};
  graphprt("-c=2 -cf=chap1-1-1-4.eps");
  xy(100*(sigma0~sigma1~sigma2),100*(mu0~mu1~mu2));

TexFile = "chap1-1-1-4.tex";
mask = 1;
let sigma_target = 0.15 0.20;
Bounds = (-3~3) .* ones(n,1);
{x,mu_x,sigma_x,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,sigma_target,2);  
phi_x = 1/gamma_x;
Results = 100*(sigma_x'|x|mu_x')|phi_x';
fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,1);
print ftosa(Results,"%lf",3,2);
print "------------------------------------------------------------------------------------";

Bounds = (0~1) .* ones(n,1);
{x,mu_x,sigma_x,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,sigma_target,2);  
phi_x = 1/gamma_x;
Results = 100*(sigma_x'|x|mu_x')|phi_x';
fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,0);
print ftosa(Results,"%lf",3,2);
print "------------------------------------------------------------------------------------";

Bounds = (0~0.40) .* ones(n,1);
{x,mu_x,sigma_x,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,sigma_target,2);  
phi_x = 1/gamma_x;
Results = 100*(sigma_x'|x|mu_x')|phi_x';
fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,0);
print ftosa(Results,"%lf",3,2);
print "------------------------------------------------------------------------------------";
