/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 26 (Optimal Black-Litterman portfolio)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';

Bounds = 0;

r = 0.03;
SR = 0.25;
let x0 = 0.40 0.30 0.20 0.10;

tau = 1;
Gamma_ = tau * cov;

let P[2,4] = 1 0 0 0
             0 1 -1 0;
let Q = 0.04 -0.01;
let Omega[2,2] = 0.05 0.0
                 0.0 0.05;
Omega = Omega^2;

tau = seqa(0,1/500,501);
nTau = rows(tau);
vol_te = zeros(nTau,1);
x_star = zeros(4,nTau);

i = 1;
do until i > nTau;
  Gamma_ = tau[i] * cov;
  {x_star[.,i],mu_x,sigma_x,vol_te[i],retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);
  i = i + 1;
endo;

target_te = 0.02;
indx = minindc(abs(vol_te-target_te));
x1 = x_star[.,indx];

proc (1) = function_tau(tau);
  local Gamma_,x_star,mu_x,sigma_x,vol_te,retcode;
  Gamma_ = tau * cov;
  {x_star,mu_x,sigma_x,vol_te,retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);
  retp(vol_te - target_te);
endp;

tau_star = Bisection(&function_tau,0,1);
Gamma_ = tau_star * cov;
{x2,mu_x,sigma_x,vol_te,retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);

print ftosa(tau_star,"tau* = %lf",3,3);
print ftosa(100*(x1~x2),"%lf",3,2);


