/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 31 (Figure 1.9)
**
*/

new;
library rpb,pgraph;

cls;

rndseed 123456;

rho = 0.70;
nS = 2000;
t = seqa(0,1/260,260);
nt = rows(t);
t1 = t;

{X1,Y1} = simulate_GBM2(100,100,0,0,0.10,0.10,rho,t1,Ns);
RX1 = ln(X1) - ln(lag1(X1));
RY1 = ln(Y1) - ln(lag1(Y1));
RX1 = trimr(RX1,1,0);
RY1 = trimr(RY1,1,0);

t = t .*. ones(2,1);
t = t + reshape(-(8/24)/260~0,2*nt,1);
t[1] = 0;
t2 = t;
indx = seqa(1,2,nt);

{X2,Y2} = simulate_GBM2(100,100,0,0,0.10,0.10,rho,t2,Ns);
X2 = X2[indx,.];
Y2 = Y2[indx+1,.];
RX2 = ln(X2) - ln(lag1(X2));
RY2 = ln(Y2) - ln(lag1(Y2));
RX2 = trimr(RX2,1,0);
RY2 = trimr(RY2,1,0);

t = seqa(0,5/260,260);
nt = rows(t);
t = t .*. ones(2,1);
t = t + reshape(-1/260~0,2*nt,1);
t[1] = 0;
t3 = t;
indx = seqa(1,2,nt);
{X3,Y3} = simulate_GBM2(100,100,0,0,0.10,0.10,rho,t3,Ns);
X3 = X3[indx,.];
Y3 = Y3[indx+1,.];
RX3 = ln(X3) - ln(lag1(X3));
RY3 = ln(Y3) - ln(lag1(Y3));
RX3 = trimr(RX3,1,0);
RY3 = trimr(RY3,1,0);

nRows = 3;
rho_hat = zeros(nS,nRows);
i = 1;
do until i > nS;
  rho_hat[i,1] = submat(corrx(RX1[.,i]~RY1[.,i]),1,2);
  rho_hat[i,2] = submat(corrx(RX2[.,i]~RY2[.,i]),1,2);
  rho_hat[i,3] = submat(corrx(RX3[.,i]~RY3[.,i]),1,2);
  i = i + 1;
endo;

R = 1|(16/24)|(4/5);
print ftosa(100*(meanc(rho_hat)~(meanc(rho_hat)./rho)~R),"%lf",3,1);

n_x = 256;
x = seqa(0.3,0.50/(n_x-1),n_x);
d = zeros(n_x,nRows);

iter = 1;
do until iter > nRows;
  {d[.,iter],F} = regKernelDensity(rho_hat[.,iter],x);
  iter = iter + 1;
endo;

d = missex(d,d .<= 1e-8);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10|10|10; _pcolor = 1|2|4;
  xlabel("\216\202r\201 (in %)");
  xtics(30,80,5,2);
  ytics(0,14,2,2);
  _plegstr = "Synchronous returns\000Daily returns (Lag = 6 hours)\000Weekly returns (Lag = 1 day)";
  _plegctl = {2 5 1.5 5.5};
  graphprt("-c=2 -cf=chap1-2-1-2.eps");
  xy(100*x,d);
