/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 33 (Figure 1.10)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data\\";
datFileName = PathName $+ "equity-indices-daily.dat";
Data = loadd(datFileName);
vnames = lower("" $+ getname(datFileName));

Data = selif(Data,Data[.,1] .>= 19990101);

Dates = Data[.,1];
nDates = rows(Dates);

x = Data[.,2:4];
R = x ./ lag1(x) - 1.0;
R = ln(x) - ln(lag1(x));

corr1 = miss(zeros(nDates,2),0);
corr2 = miss(zeros(nDates,2),0);
corr3 = miss(zeros(nDates,2),0);

dt = 260;

t = dt + 1;
do until t > nDates;
  tmp = R[t-dt:t,.];
  tmp = packr(tmp);
//  tmp = tmp - meanc(tmp)';
  rho = corrx(tmp);
  corr1[t,1] = rho[1,2];
  corr2[t,1] = rho[1,3];
  corr3[t,1] = rho[2,3];

  cov1 = meanc(tmp[.,1]^2);
  cov2 = meanc(tmp[.,2]^2);
  cov3 = meanc(tmp[.,3]^2);
  cov12 = meanc(tmp[.,1] .* tmp[.,2]) + meanc(packr(lag1(tmp[.,1]) .* tmp[.,2]));
  cov13 = meanc(tmp[.,1] .* tmp[.,3]) + meanc(packr(lag1(tmp[.,1]) .* tmp[.,3]));
  cov23 = meanc(tmp[.,2] .* tmp[.,3]) + meanc(packr(lag1(tmp[.,2]) .* tmp[.,3]));

  corr1[t,2] = cov12 ./ sqrt(cov1 .* cov2);
  corr2[t,2] = cov13 ./ sqrt(cov1 .* cov3);
  corr3[t,2] = cov23 ./ sqrt(cov2 .* cov3);

  t = t + 1;
endo;

indx = selif(seqa(1,1,nDates),(Dates .>= 20000101) .and (Dates .<= 20121231));
ratio = (corr1[.,1]./corr1[.,2])~(corr2[.,1]./corr2[.,2])~(corr3[.,1]./corr3[.,2]);
print ftosa(100*meanc(ratio[indx,.]),"%lf",3,2);

indx = selif(seqa(1,1,nDates),(Dates .>= 20000101) .and (Dates .<= 20121231));
step = 22.5;
indx = indx[seqa(1,step,rows(indx)/step)];
Dates = Dates[indx];
t = seqa(1,1,rows(indx));

graphset;  
  window3u;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; _pgrid = {2,1};
  _plwidth = 10; _pltype = 6|3|1; _pcolor = 1|4;
  _paxht = 0.22; _pnumht = 0.20; _ptitlht = 0.25;
  xlabel("\216t");
  ytics(0,100,20,2);
  ylabel("\216\202r\201 (in %)");
  xtics(1,rows(t),2*12,2);
  let xlab = "2000" "02" "04" "06" "08" "10" "12";
  asclabel(xlab,0);

  setwind(1);
    title("\216SPX / SXXE");
    _plegstr = "Empirical estimator\000Hayashi-Yoshida estimator";
    _plegctl = {2 6 6 1.5};
    xy(t,100*corr1[indx,.]);

    _plegctl = 0;

  setwind(2);
    title("\216SPX / TPX");
    xy(t,100*corr2[indx,.]);

  setwind(3);
    title("\216SXXE / TPX");
    xy(t,100*corr3[indx,.]);

    graphprt("-c=2 -cf=chap1-2-1-3.eps");

  endwind;
