/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 39 (Results 1.19 & 1.21)
**
*/

new;
library rpb,pgraph,optmum;

PathName = "data\\";
datFileName = PathName $+ "equity-indices-weekly.dat";
Data = loadd(datFileName);
vnames = lower("" $+ getname(datFileName));

BeginDate = 20050101;
LastDate = 20111231;
Data = selif(Data,(Data[.,1] .>= BeginDate) .and (Data[.,1] .<= LastDate));

let string selected_names = "spx" "sxxe" "ukx" "tpx" "bovespa" "rts" "nifty" "hsi";
let Factors               = 1     1      1     1     2         2     2       2;

indx = indsav(selected_names,vnames);
Dates = Data[.,1];
x = Data[.,indx];
R = x ./ lag1(x) - 1.0;
R = packr(R);

C0 = corrx(R);
rho = diagrv(miss(lowmat(C0),0),error(0));

rho0 = meanc(packr(vec(rho)));

_reg_PrintIters = 0;
{rho1,C1} = regCorr1F(R);

_reg_cov = 0;
_regCorrMF_Bound = 0.50;
{rho_c,rho_f,C2,cor} = regCorrMF(R,1,Factors,0);

cls;

print ftosa(100*rho0,"%lf",3,2);
print ftosa(100*rho1,"%lf",3,2);

print ftosa(miss(lowmat(C2),0),"%lf",3,2);

declare matrix C4,C5,C6,A4,A5,A6;

p = 1;
do until p > 3;
  {A,D} = regLinearFactorModel(C0,p);
  if p == 1;
    let sgn = 1;
  elseif p == 2;
    let sgn = 1 1;
  elseif p == 3;
    let sgn = -1 1 1;
  endif;
  A = A .* sgn';
  V = A*A' + D;
  s = sqrt(diag(V));
  C = V ./ s ./ s';
  call invpd(C);
  call varput(C,itos_rpb("C",p+3));
  call varput(A,itos_rpb("A",p+3));
  p = p + 1;
endo;

print ftosa(miss(lowmat(C5),0),"%lf",3,2);
print ftosa(A5,"%lf",1,4);
