/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 46 (Figure 1.16)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.09 0.07 0.06;
let sigma = 0.04 0.15 0.05 0.10;
let rho =   1.00 
            0.10  1.00
            0.40  0.20  1.00
           -0.10 -0.10 -0.20 1.00;
rho = xpnd(rho);
Cov = rho .* sigma .* sigma';

Bounds = 0~1;

gamma_ = seqa(-1,0.005,300);
{x,er,risk,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,gamma_,0);  

nS = 100;
let gamma_ = 0.00 0.025 0.05 0.075 0.10 0.15 0.20 0.30 0.40 0.50;
nGamma = rows(gamma_);
all_er = zeros(nGamma,nS);
all_risk = zeros(nGamma,nS);

rndseed 123;

gamma_ = gamma_ / 2; // To obtain exactly Figure 1.16 on Page 46

i = 1;
do until i > nS;
  r = rndmn(mu,Cov,260);
  c = vcx(r);
  m = meanc(r);
  {x,all_er[.,i],all_risk[.,i],gamma_x,retcode} = Compute_MVO_Portfolio(m,c,1,0,0,0,Bounds,gamma_,0);
  i = i + 1;
endo;

all_er = vecr(all_er);
all_risk = vecr(all_risk);
pcolor = 1|2|3|4|5|6|7|8|13|12; 
psym = seqa(1,1,nGamma); 
psym = 1|8|2|9|3|10|4|11|5|12;
all_color = vecr(pcolor .* ones(1,nS));
all_sym = vecr(psym .* ones(1,nS));
np = rows(all_er);

graphset;
  begwind;
  makewind(9,6.855,0,0,1);
  makewind(9,6.855,0,0,1);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected Return (in %)");
  xtics(3,6,0.5,2);
  ytics(5,7.5,0.5,2);

  setwind(1);
    _psym = 100*(all_risk~all_er)~all_sym~ones(np,1).*2~all_color~ones(np,1).*(1~2);
    xy(100*Risk,100*ER);

  setwind(2);
    _psym = 0; _pcolor = 0|0; _plwidth = 12;
    xy(100*Risk,100*ER);

    graphprt("-c=2 -cf=chap1-2-3-2.eps");

  endwind;

