/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 112 (Table 2.12 and Figure 2.6)
**
*/

new;
library rpb,pgraph;

cls;

/*
**> The case when one risk budget is set to zero
**
*/

msym ".";

all_results = miss(zeros(3*4,4),0);

/*
**> Negative correlations
**
*/

let sigma = 0.20 0.10 0.05;
let rho =   1.00
            0.50   1.00
           -0.25  -0.25  1.00;
rho = xpnd(rho);
cov = rho .* sigma .* sigma';
call invpd(cov);
let b = 0.50 0.50 0.00;
n = rows(cov);

i = 1;
{x,vol,mr,rc,prc} = Compute_RB_Portfolio(0,cov,b);
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
all_results[1+(i-1)*3:i*3,1:3] = 100*(x~mr~prc)';
all_results[1+(i-1)*3+1,4] = 100*vol;

i = 2;
indx = selif(seqa(1,1,n), b ./= 0);
{x,vol,mr,rc,prc} = Compute_RB_Portfolio(0,cov[indx,indx],b[indx]);
tmp = zeros(n,1);
tmp[indx] = x;
x = tmp;
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
all_results[1+(i-1)*3:i*3,1:3] = 100*(x~mr~prc)';
all_results[1+(i-1)*3+1,4] = 100*vol;

let b = 0.495 0.495 0.01;
i = 3;
{x,vol,mr,rc,prc} = Compute_RB_Portfolio(0,cov,b);
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
all_results[1+(i-1)*3:i*3,1:3] = 100*(x~mr~prc)';
all_results[1+(i-1)*3+1,4] = 100*vol;

let b = 0.50 0.50 0.00;
let x = 0.50 0.50 0.00;
eps = seqa(-1.00,2.00/100,101);

neps = rows(eps);
vol_results = miss(zeros(neps,2),0);
j = 1;
do until j > neps;
  x[3] = eps[j]; 
  vol_results[j,1] = sqrt(x'cov*x);
  j = j + 1;
endo;

/*
**> Positive correlations
**
*/

let rho =   1.00
            0.50   1.00
            0.25   0.25  1.00;
rho = xpnd(rho);
cov = rho .* sigma .* sigma';
call invpd(cov);

i = 4;
{x,vol,mr,rc,prc} = Compute_RB_Portfolio(0,cov,b);
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
all_results[1+(i-1)*3:i*3,1:3] = 100*(x~mr~prc)';
all_results[1+(i-1)*3+1,4] = 100*vol;

let x = 0.50 0.50 0.00;
j = 1;
do until j > neps;
  x[3] = eps[j]; 
  vol_results[j,2] = sqrt(x'cov*x);
  j = j + 1;
endo;

cls;
print ftosa(all_results[1:3,.],"%lf",3,2);
print ftosa(all_results[4:6,.],"%lf",3,2);
print ftosa(all_results[7:9,.],"%lf",3,2);
print ftosa(all_results[10:12,.],"%lf",3,2);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; _pcross = 1;
  _pltype = 6|3; _pcolor = 1|4; _plwidth = 10;
  xtics(-100,100,50,2);
  ytics(12.5,14,0.5,2);
  xlabel("\216x]3[ (in %)");
  ylabel("\216\202s\201(x) (in %)");
  _plegstr = "\202r\201]1,3[ = \202r\201]2,3[ = -25%\000\202r\201]1,3[ = \202r\201]2,3[ = +25%";
  _plegctl = {2 5 5.5 3};
  graphprt("-c=2 -cf=chap2-2-2-4.eps");
  xy(100*eps,100*vol_results);
