/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 117 (Tables 2.16 and 2.17)
**
*/

new;
library rpb,pgraph;

cls;

let mu    = 0.08 0.08 0.05; 
let sigma = 0.20 0.21 0.10;

let rhoVectors = 0.80 0.70 0.90 0.80 0.90 0.80;
let sigma2     = 0.21 0.21 0.21 0.18 0.18 0.21;
let mu1        = 0.08 0.08 0.08 0.08 0.08 0.09;
n = rows(sigma);

target_vol = 0.15;

nIters = rows(rhoVectors);
all_MVO = zeros(n,nIters);
all_RB = zeros(n,nIters);

/*
**> Generic case
**
*/

rho = 0.80;
rho = diagrv(rho*ones(n,n),1.0);
cov = rho .* sigma .* sigma';
call invpd(cov);
{x,er,vol,gamma_,retcode} = Compute_MVO_Portfolio(mu,cov,ones(1,n),1,0,0,(0~1).*ones(n,1),target_vol,2);
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
b = rc;

Results = x~mr~rc~prc;

cov0 = cov;

i = 1;
do until i > nIters;
  rho = rhoVectors[i];
  sigma[2] = sigma2[i];
  mu[1] = mu1[i];
  rho = diagrv(rho*ones(n,n),1.0);
  cov = rho .* sigma .* sigma';
  call invpd(cov);
  {all_MVO[.,i],er,vol,gamma_,retcode} = Compute_MVO_Portfolio(mu,cov,ones(1,n),1,0,0,(0~1).*ones(n,1),target_vol,2);
  {all_RB[.,i],vol,mr,rc,prc} = Compute_RB_Portfolio(0,cov,b); 
  i = i + 1;
endo;

cls;

print ftosa(100*Results,"%lf",3,1);

print ftosa(100*all_MVO,"%lf",3,1);

print ftosa(100*all_RB,"%lf",3,1);

