/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 125 (Figure 2.8)
**
*/

new;
library rpb,pgraph;

let sigma = 0.05 0.07 0.09 0.10 0.15;

let SR = 0.4 1.0 0.40 1.00;
let rho1 = 1.0
           0.5 1.0
           0.25 0.25 1.0
           0.00 0.00 0.00 1.0
           -0.25 -0.25 -0.25 0.00 1.00;
rho1 = xpnd(rho1);
rho2 = rho1;
           
let r0 = 0.02 0.02 0.04 0.04;

gamma_ = seqa(-5,5/2000,10001);
nGamma = rows(gamma_);
p_mu = zeros(nGamma,4);
p_sigma = zeros(nGamma,4);
stats = zeros(4,4);

declare matrix psym1,psym2,psym3,psym4;

iter = 1;
do until iter > 4;
  er = r0[iter] + sR[iter] * sigma;
  n = rows(er);
  if (iter == 1) or (iter == 3);
    cor = rho1;
  else;
    cor = rho2;
  endif;

  call invpd(cor);
  cov = sigma .* sigma' .* cor;

  {x,p_mu[.,iter],p_sigma[.,iter],gamma_,retcode} = Compute_MVO_Portfolio(er,cov,1,1,0,0,1,gamma_,0);

  {x0,sigma0,mr,rc,prc} = Compute_ERC_Portfolio(0,cov);
  mu0 = x0'er;

  sh = (p_mu[.,iter]-r0[iter])./p_sigma[.,iter];
  indx = maxindc(sh);
  x1 = x[.,indx];
  mu1 = x1'er;
  sigma1 = sqrt(x1'cov*x1);

  stats[.,iter] = sigma0|mu0|sigma1|mu1;

  psym = 100*sigma0~100*mu0~8~4.0~4~1~10 |
         100*sigma1~100*mu1~11~7.0~1~1~10 ;
  str = itos_rpb("psym",iter);
  call varput(psym,str);

  iter = iter + 1;
endo;

let ymin = 4 6 6 8;
let ymax = 8 16 10 18;
let step = 1 2 1 2;

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma"); _pgrid = {1,1};
  _pdate = ""; _pnum = 2; _pframe = 0; _pcolor = 1|4;
  _pltype = 6|3; _plwidth = 10;
  xlabel("\216\202s\201(x) (in %)");
  ylabel("\216\202m\201(x) (in %)");
  xtics(2,10,2,0);
  _paxht = 0.22; _pnumht = 0.22; _ptitlht = 0.25;

  iter = 1;
  do until iter > 4;
    ytics(ymin[iter],ymax[iter],step[iter],0);
    setwind(iter);
      str = ftos(SR[iter],"\216SR = %lf",3,2) $+ ftos(100*r0[iter]," - r = %lf",1,0) $+ "%";
      title(str);
      _pline = 1~3~0~100*r0[iter]~100*2*stats[3,iter]~100*(2*stats[4,iter]-r0[iter])~1~0~10;
      str = itos_rpb("psym",iter);
      _psym = varget(str);
      xy(100*p_sigma[.,iter],100*p_mu[.,iter]);      

    iter = iter + 1;
  endo;

  graphprt("-c=2 -cf=chap2-3-3-2.eps");

  endwind;
