/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 134 (Table 2.24 & Figure 2.10)
**
*/

new;
library rpb,pgraph;

cls;

__ouput = 0;

let mu =    0.10 0.05 0.05;
let sigma = 0.30 0.20 0.15;
let rho = 1.00
          0.80 1.00
          0.50 0.30 1.00;
rho = xpnd(rho);
cov = rho .* sigma .* sigma';
n = rows(cov);

x0 = ones(n,1)/n;
let b = 0.50 0.20 0.30;

// Minimum risk portfolio
bounds = (0~1) .* ones(n,1);
{x,vol_min,retcode} = Compute_MV_Portfolio(cov,ones(1,n),1,0,0,bounds);
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
result_mv = (x~prc)|(error(0)~vol);

vol_mv = vol;

x = b;
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
result_wb = (x~prc)|(error(0)~vol);

{x,vol,mr,rc,prc} = Compute_RB_Portfolio(x0,cov,b);
result_rb = (x~prc)|(error(0)~vol);

cls;

nIters = 16;
vol = zeros(nIters,1);
msgstr = "" $+ zeros(nIters,1);

iter = 1;
x = b;
{vol[iter],mr,rc,prc} = Compute_Risk_Contribution(x,cov);
result = (x~prc)|(error(0)~vol[iter]);
call varput(result,"result0");
msgstr[iter] = itos_rpb("\202s\201(x[(",iter - 1) $+ ftos(100*vol[iter],")]) = %lf",1,2) $+ "%";

iter = 2;
do until iter > nIters;
  rb = x;
  {x,vol[iter],mr,rc,prc} = Compute_RB_Portfolio(x0,cov,rb);
  result = (x~prc)|(error(0)~vol[iter]);
  call varput(result,itos_rpb("result",iter-1));
  msgstr[iter] = itos_rpb("\202s\201(x[(",iter - 1) $+ ftos(100*vol[iter],")]) = %lf",1,2) $+ "%";
  iter = iter + 1;
endo;

print ftosa(100*result_wb,"%lf",3,2);
print "-----------------------------------------------------------------------";

print ftosa(100*result_rb,"%lf",3,2);
print "-----------------------------------------------------------------------";

print ftosa(100*result_mv,"%lf",3,2);
print "-----------------------------------------------------------------------";

result = varget("result0") ~ varget("result1");
print ftosa(100*result,"%lf",3,2);

result = varget("result2") ~ varget("result3");
print ftosa(100*result,"%lf",3,2);

result = varget("result4") ~ varget("result5");
print ftosa(100*result,"%lf",3,2);

result = varget("result6") ~ result_mv;
print ftosa(100*result,"%lf",3,2);

print "-----------------------------------------------------------------------";

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plctrl = -1; _pcolor = 1; _pstype = 8;
  _pline = 1~3~0~100*vol_min~nIters~100*vol_min~1~2~10 ;
  xtics(0,nIters-1,1,0);
  xlabel("\216Iteration k");
  ylabel("\216Volatility (in %)");
  _pmsgstr = MakeMsgStr(msgstr[1 2 3 4 5 11]) $+ "\000\202s\201(x[(mv)]) = " $+ ftos(100*vol_mv,"%lf",1,2) $+ "%";
  _pmsgctl = 0.30~100*vol[1]~0.12~0~1~4~7 |
             1.30~100*vol[2]~0.12~0~1~4~7 |
             2.30~100*vol[3]~0.12~0~1~4~7 |
             3.30~100*vol[4]~0.12~0~1~4~7 |
             4.30~100*vol[5]~0.12~0~1~4~7 |
             10.10~100*vol[11]+0.15~0.12~0~1~4~7 |
             0.40~100*vol_mv+0.15~0.12~0~1~2~7 ;
  graphprt("-c=2 -cf=chap2-4-2-1.eps");
  xy(seqa(0,1,nIters),100*vol);


