/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 183 (Figure 3.7)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data//";
datFileName = Pathname $+ "sx5e-info";
loads info = ^datFileName;
datFileName = Pathname $+ "sx5e-weights";
Composition = loadd(datFileName);
datFileName = Pathname $+ "sx5e-prices";
Prices = loadd(datFileName);
nStocks = cols(Prices) - 2;

Weights = Composition[.,2:cols(Composition)]/100;
maxWeights_CW = maxc(Weights');

Rebalancing_Dates = Composition[.,1];
Rebalancing_Dates[1] = 19920101;
nRD = rows(Rebalancing_Dates);
nAssets_CW = 50 * ones(nRD,1);

Dates = Prices[.,1];
Index = Prices[.,2];
Prices = Prices[.,3:2+nStocks];

EndDate = 20120928;

Lag_CovMatrix = 260;
FileName = itos_rpb("Results\\sx5e-Weights-basic-",Lag_CovMatrix);
loadm Computed_Weights = ^FileName;
FileName = itos_rpb("Results\\sx5e-RC-basic-",Lag_CovMatrix);
loadm Computed_RC = ^FileName;

nAW = submat(getorders(Computed_Weights),1,0);

q = seqa(1,1,50)/50;
Lorenz_W = zeros(50,nAW+1);
Lorenz_RC = zeros(50,nAW);

Begin_Iter = 13;
i = Begin_Iter;
do until i > nRD;

  tmp_Computed_Weights = array3d_get_y(Computed_Weights,i)';
  tmp_Index_Weights = Weights[i,.]';
  tmp_Weights = tmp_Computed_Weights~tmp_Index_Weights;
  indx = selif(seqa(1,1,nStocks),tmp_Index_Weights[.,1] ./= 0.0);
  tmp_Weights = tmp_Weights[indx,.];

  tmp_Computed_RC = array3d_get_y(Computed_RC,i)';
  tmp_RC = tmp_Computed_RC;
  tmp_RC = tmp_RC[indx,.];

  j = 1;
  do until j > nAW;
    tmp = tmp_Weights[.,j];
    tmp = rev(sortc(tmp,1));
    tmp = cumsumc(tmp);
    tmp = tmp/tmp[rows(tmp)];
    x = seqa(1,1,rows(tmp))/rows(tmp);
    y = fspline(csspline(x,tmp,1,1),q);
    Lorenz_W[.,j] = Lorenz_W[.,j] + y;

    tmp = tmp_RC[.,j];
    tmp = rev(sortc(tmp,1));
    tmp = cumsumc(tmp);
    x = seqa(1,1,rows(tmp))/rows(tmp);
    y = fspline(csspline(x,tmp,1,1),q);
    Lorenz_RC[.,j] = Lorenz_RC[.,j] + y;

    j = j + 1;
  endo; 

// test
    tmp = tmp_Weights[.,j];
    tmp = rev(sortc(tmp,1));
    tmp = cumsumc(tmp);
    x = seqa(1,1,rows(tmp))/rows(tmp);
    y = fspline(csspline(x,tmp,1,1),q);
    Lorenz_W[.,j] = Lorenz_W[.,j] + y;
 
  i = i + 1;
endo;

Lorenz_W = Lorenz_W/(nRD-Begin_Iter+1);
Lorenz_W = zeros(1,nAW+1) | Lorenz_W;

Lorenz_RC = Lorenz_RC/(nRD-Begin_Iter+1);
Lorenz_RC = zeros(1,nAW) | Lorenz_RC;

q = 0|q;

FileName = itos_rpb("Results\\sx5e-Weights-",Lag_CovMatrix);
loadm Computed_Weights = ^FileName;

nAW = 8;
maxWeights = zeros(nRD,nAW);
nAssets = zeros(nRD,nAW);
lower_bound = 0.00001;

i = 1;
do until i > nAW;
  Weights = array3d_get_x(Computed_Weights,i)';
  m = miss(maxc(Weights),0);
  maxWeights[.,i] = m;
  nAssets[.,i] = sumr(Weights' .> lower_bound);
  i = i + 1;
endo;

nAssets[.,2 4] = 50 * ones(nRD,2);

nAssets = miss(nAssets,0);

graphset;
  begwind;
  window(2,2,0);  
    _pdate = ""; _pnum = 2; _pframe = 0;
    _pltype = 1|6|3|6|6; _plwidth = 10; _pcolor = 5|1|2|3|4; 
    _plctrl = 0|10|0|0|10; _pstype = 0|8|0|0|9; _psymsiz = 5;
    ytics(0,100,20,2);
    xtics(0,100,20,2);
    _plegstr = "CW\000EW\000MV\000MDP\000ERC";
    _paxht = 0.25; _pnumht = 0.225; _ptitlht = 0.25;

  setwind(1);
    title("\216Weights");
    xy(100*q,100*Lorenz_W[.,9 4 1 3 2]);

  setwind(2);
    title("\216Risk contributions");
    _plegctl = {2 6 6.5 1};
    xy(100*q,100*Lorenz_RC[.,9 4 1 3 2]);

  _plctrl = 0;

  setwind(3);
    title("\216Maximum weight");
    _pboxctl = { .25 0 1 }; _plwidth = 10;
    xtics(0,5,1,0);
    let xlab = "CW" "EW" "MV" "MDP" "ERC";
    asclabel(xlab,0);
    _plegctl = 0;
    x = packr(maxWeights_CW~maxWeights[.,4 1 3 2]);
    call box(0,100*x);

  setwind(4);
    title("\216Number of invested stocks");
    ytics(0,60,10,0);
    x = packr(nAssets_CW~nAssets[.,4 1 3 2]);
    call box(0,x);

    graphprt("-c=2 -cf=chap3-3-1-6.eps");
  endwind;

cls;

x = maxWeights_CW~maxWeights[.,4 1 3 2];
indx = maxindc(x);
print ftosa(Rebalancing_Dates[indx]~100*subvec(x',indx),"%lf",3,2);

x = packr(nAssets_CW~nAssets[.,4 1 3 2]);
print ftosa(minc(x)~median(x)~maxc(x),"%lf",3,0);

