/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 197 (Figure 4.2)
**
*/

new;
library rpb,pgraph;

cls;

if 0;

  loadm ns_frf = data\zc-ns-frf;
  NS_Dates = ns_frf[.,1];
  NS_Coeffs = ns_frf[.,2 3 4 5];
  R_0 = NS_coeffs[.,1] + NS_coeffs[.,2];
  R_Infinity = NS_Coeffs[.,1];
  tau = seqa(1/24,1/24,24*10);
  Yield_Rate = NelsonSiegel_Spot_Rate(NS_Coeffs',tau)';

else;

  datFileName = "data\\zc-ir-2009.dat";
  data = loadd(datFileName);
  data = selif(data,(data[.,1] .>= 20030101) .and (data[.,1] .<= 20120630));
  Yield_Rate = data[.,2:cols(data)]/100;
  loadm maturity = data\zc-tau;
  tau = seqa(0,1,10);
  tau = maturity;
  indx = indnv(tau,Maturity);
  Yield_Rate = Yield_Rate[.,indx];  

endif;

c = corrx(Yield_Rate);

__output = 0;
{va,ve,pca} = regPCA(c,5);

maturity = seqa(1/24,1/24,24*20);
factors = zeros(rows(maturity),3);
i = 1;
do until i > 3;
  factors[.,i] = fspline(csspline(tau,ve[.,i],1,0.90),maturity);
  i = i + 1;
endo;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2|4; _pcross = 1;
//  ytics(-0.15,0.20,0.05,2);
  xtics(0,10,1,2);
  xlabel("\216T (in years)");
  _plegstr = "Factor 1\000Factor 2\000Factor 3";
  _plegctl = {2 5 6.4 1.20};
  graphprt("-c=2 -cf=chap4-2-1-2.eps");
  xy(maturity,factors);


