/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 218 (Figure 4.8)
**
*/

new;
library rpb,pgraph;

cls;

_Bond_Continuous_Time = 1;
_bond_window = 2;

__output = 0;

loadm maturity = data\zc-tau;

if 0;
  datFileName = "data\\zc-ir-2003.dat";
  data = loadd(datFileName);
  data = selif(data,(data[.,1] .>= 20030101) .and (data[.,1] .<= 20120630));
  FileName = "results/zc-cov-2003";
else;
  datFileName = "data\\zc-ir-2009.dat";
  data = loadd(datFileName);
  data = selif(data,(data[.,1] .>= 20100630) .and (data[.,1] .<= 20120630));
  FileName = "results/zc-cov-2010";
endif;

Dates = data[.,1];
nDates = rows(Dates);
Yield_Rate = data[.,2:cols(data)]/100;
tau = Maturity;
tau = seqa(1,1,10);

indx = indnv(tau,Maturity);
Yield_Rate = Yield_Rate[.,indx];

Rt = Yield_Rate[rows(Yield_Rate),.]';

dt = 10;

// PCA Factors

Factors = Yield_Rate - lagn(Yield_Rate,dt);
Factors = packr(Factors);
cov = vcx(Factors);
save ^FileName = cov;
{va,ve,pca} = regPCA(cov,5);
PCA_tau = seqa(1/24,1/24,24*20);
PCA_Factors = zeros(rows(PCA_tau),3);
i = 1;
do until i > 3;
  PCA_Factors[.,i] = fspline(csspline(tau,ve[.,i],1,0.90),PCA_tau);
  i = i + 1;
endo;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2|4; _pcross = 1;
  xtics(0,10,1,2);
  xlabel("\216T (in years)");
  _plegstr = "Factor 1\000Factor 2\000Factor 3";
  _plegctl = {2 5 6.4 1.20};
  xy(PCA_tau,PCA_Factors);

// PCA Factors

dR = Yield_Rate - lagn(Yield_Rate,dt);
t = tau;

dR = packr(dR);
Sigma = vcx(dR);
mu = meanc(dR);

let indx_Factors = 1 2 3;
nFactors = rows(indx_Factors);

{va,ve} = eighv(Sigma);
va = rev(va); 
ve = ve[.,seqa(cols(ve),-1,cols(ve))];
va = va[indx_factors]; ve = ve[.,indx_factors];
Omega = diagrv(eye(nFactors),va);
A = -ve;
D = Sigma - A*Omega*A';
P = pinv(A);

nZC = rows(t);
Nt = 100*ones(nZC,1);

Bt = exp(-t .* Rt);
Dt = t;

nB = ones(nZC,1);  // Number of ZC Bonds

Pt = Nt .* Bt;
wB =  (nB .* Pt)/sumc(nB .* Pt);
delta = - Dt .* Bt .* nB .* Nt;
wR = delta;

Long_Leg = sumc(selif(nB .* Pt, nB .>= 0));
w = (nB .* Pt)/Long_Leg;

{vol,mr,rc,prc} = Compute_Risk_Contribution(wR,Sigma);
RC_A = prc;

{vol,y,mr,rc,prc} = Compute_RC_Factor(wR,Sigma,A,Omega,D);
RC_F = prc[1:3]|sumc(prc[4:nZC]);

print ftosa(100*RC_F,"%lf",3,2);

graphset;
  window3d;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pnumht = 0.22; _paxht = 0.20; _ptitlht = 0.275;

  setwind(1);
    title("\216\202u\201]i[");
    ytics(0,12,2,2);
    _pbartyp = {6 1};
    bar(0,100*w);

  setwind(2);
    title("\216RC[*]]i[ (in %)");
    ytics(0,20,5,2);
    _pbartyp = {6 2};
    bar(0,100*RC_A);

  setwind(3);
    title("\216RC[*](F]j[) (in %)");
    ytics(0,100,20,2);
    let xlab = "1" "2" "3" "4+";
    asclabel(xlab,0);
    _pbartyp = {6 4};
    bar(0,100*RC_F);

    graphprt("-c=2 -cf=chap4-3-1-1.eps");

  endwind;

