/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 221 (Figure 4.13)
**
*/

new;
library rpb,pgraph;

cls;

_Bond_Continuous_Time = 1;
_bond_window = 2;

__output = 0;

if 1;
  FileName = "Results\\zc-cov-2003";
else;
  FileName = "Results\\zc-cov-2010";
endif;

loadm maturity = data\zc-tau;

datFileName = "data\\zc-ir-2009.dat";
data = loadd(datFileName);
data = selif(data,(data[.,1] .>= 20100630) .and (data[.,1] .<= 20120630));
Dates = data[.,1];
nDates = rows(Dates);
Yield_Rate = data[.,2:cols(data)]/100;
tau = Maturity;
tau = seqa(1,1,10);

indx = indnv(tau,Maturity);
Yield_Rate = Yield_Rate[.,indx];

Rt = Yield_Rate[rows(Yield_Rate),.]';

// PCA Factors

dt = 10;
dR = Yield_Rate - lagn(Yield_Rate,dt);
t = tau;

dR = packr(dR);
Sigma = vcx(dR);
loadm Sigma = ^FileName;
mu = meanc(dR);

let indx_Factors = 1 2 3;
nFactors = rows(indx_Factors);

{va,ve} = eighv(Sigma);
va = rev(va); 
ve = ve[.,seqa(cols(ve),-1,cols(ve))];
va = va[indx_factors]; ve = ve[.,indx_factors];
Omega = diagrv(eye(nFactors),va);
A = -ve;
D = Sigma - A*Omega*A';
P = pinv(A);

t = 2;
Ct = 100;
t1 = t; C1 = Ct;

t = 5;
Ct = 100;
t2 = t; C2 = Ct;

t = 10;
Ct = 100;
t3 = t; C3 = Ct;

let params = 0.05 -0.05 0.06 10;
fn NelsonSiegel_Bond_Price(t,Ct,params) = Bond_Price(t,Ct,NelsonSiegel_Spot_Rate(params,t));

t = t1; Ct = C1;
P = NelsonSiegel_Bond_Price(t,Ct,params);
YTM = Bond_YTM(t,Ct,P);
S = Bond_Duration(t,Ct,YTM);
P1 = P; YTM1 = YTM; S1 = S;

t = t2; Ct = C2;
P = NelsonSiegel_Bond_Price(t,Ct,params);
YTM = Bond_YTM(t,Ct,P);
S = Bond_Duration(t,Ct,YTM);
P2 = P; YTM2 = YTM; S2 = S;

t = t3; Ct = C3;
P = NelsonSiegel_Bond_Price(t,Ct,params);
YTM = Bond_YTM(t,Ct,P);
S = Bond_Duration(t,Ct,YTM);
P3 = P; YTM3 = YTM; S3 = S;

data = P1~100*YTM1~S1 |
       P2~100*YTM2~S2 |
       P3~100*YTM3~S3 ;

print ftosa(data,"%lf",3,2);

DV01 = S1|S2|S3;
Tau = 2|5|10;
Price = P1|P2|P3;

beta_ = 0.5;

{w1,w2,w3,w4} = Barbell_Calibrate(DV01,Price,Tau,beta_);
w = w1~w2~w3~w4;
print ftosa(w,"%lf",3,3);

nZC = 3;
t = 2|5|10;
Rt = NelsonSiegel_Spot_Rate(params,t);
Nt = 100*ones(nZC,1);
Bt = exp(-t .* Rt);
Dt = t;
Pt = Nt .* Bt;

RC_F = zeros(4,4);

i = 1;
do until i > 4;
  nB = w[.,i];
  wB =  (nB .* Pt)/sumc(nB .* Pt);
  delta = - Dt .* Bt .* nB .* Nt;
  wR = delta;
  {vol,y,mr,rc,prc} = Compute_RC_Factor(wR,Sigma[t,t],A[t,.],Omega,D[t,.]);
  RC_F[1:3,i] = prc[1:3];
  i = i + 1;
endo;

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pnumht = 0.22; _paxht = 0.20; _ptitlht = 0.275;

  ytics(0,100,20,2);
  let xlab = "1" "2" "3" "4+";
  asclabel(xlab,0);

  setwind(1);
    title("\21650/50");
    _pbartyp = {6 1};
    bar(0,100*RC_F[.,1]);

  setwind(2);
    title("\216Cash-neutral");
    _pbartyp = {6 2};
    bar(0,100*RC_F[.,2]);

  setwind(3);
    title("\216Maturity-weighted");
    _pbartyp = {6 4};
    bar(0,100*RC_F[.,3]);

  setwind(4);
    title("\216Regression-weighted");
    _pbartyp = {6 5};
    bar(0,100*RC_F[.,4]);

    graphprt("-c=2 -cf=chap4-3-1-7.eps");

  endwind;
