/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 224 (Table 4.12)
**
*/

new;
library rpb,pgraph,optmum;

cls;

FileName = "Data//sovereign-bonds.fmt";
loadm databuf = ^FileName;
Dates = vread(databuf,"dates");
Spread = vread(databuf,"spread");
Spread_Greece = vread(databuf,"sp-gr");
Duration = vread(databuf,"duration");
Weights_EGBI = vread(databuf,"w(egbi)");
Weights_GDP = vread(databuf,"w(gdp)");
Weights_DEBT = vread(databuf,"w(debt)");
EGBI_index = vread(databuf,"index");
EGBI_Subindex = vread(databuf,"subindex");
Libor = vread(databuf,"libor");

Begin_Date = 20071231;
End_Date = 20120630;
nDates = rows(Dates);

nCountries = cols(Spread);

Spread = selif(Spread, (Dates .>= Begin_Date) .and (Dates .<= End_Date));
Dates = selif(Dates, (Dates .>= Begin_Date) .and (Dates .<= End_Date));

let sv = 1 1;
Results = zeros(6,nCountries);

_reg_cov = 2;

i = 1;
do until i > nCountries;
  {t,St} = packr2(dates,Spread[.,i]);
  S_lag = lag1(St);
  dS = St - S_lag;
  dt = etdays2(lag1(t),t)/365;
  {theta,stderr,cov,logl} = regML(0,0,&mlProc,sv,0,0);
  tstudent = (theta[1] - (0.0|1.0))/stderr[1];
  df = rows(S_lag) - 2;
  pvalue = 2*cdftc(abs(tstudent),df);
  Results[.,i] = theta[1]|stderr[1]|tstudent|pvalue; 
  i = i + 1;
endo;

cls;
Results = Results ~ meanc(Results');
print ftosa(Results[1:2,1:6] .* (1|100),"%lf",3,3);
print ftosa(Results[1:2,7:12] .* (1|100),"%lf",3,3);

proc mlProc(theta);
  local beta_,sigma_S,sigma,logl;

  beta_ = sqrt(theta[1]^2);
  sigma_S = sqrt(theta[2]^2);
  sigma = sigma_S .* (S_lag^beta_) .* sqrt(dt);
  logl = - 0.5 * ln(2*pi) - 0.5 * ln(sigma^2) - 0.5 * (dS^2)./(sigma^2);

  retp(logl);
endp;

