/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 263-265 (Table 5.7 and Figures 5.8, 5.9, 5.10 & 5.11)
**
*/

new;
library rpb,pgraph;

cls;

datPathName = "Data\\";
datFileName = datPathName $+ "djcs-hf";

resPathName = "Results\\DJCS-3F";

Data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

let string Selected_Names[10,1] = CA SB EM EMN ED FIA GM LSE MF MS;

indx = indsav(Selected_Names,vnames);
nAssets = rows(indx);

Dates = Data[.,1];
nDates = rows(Dates);
t = seqa(2004 + 9/12,1/12,nDates);
Index_NAV = Data[.,2];
SubIndex_NAV = Data[.,indx];

R_SubIndex = SubIndex_NAV ./ lag1(SubIndex_NAV) - 1.0;
nLags = 24;

let x0 = 3 1 6 4 22 5 14 24 6 13;
x0 = x0 / sumc(x0);
index_weights = x0' .* ones(nDates,1);

nIters = 7;
Asset_Weights = arrayinit(nIters|nDates|nAssets,error(0));

let indx_Factors = 1 2 3;
nFactors = rows(indx_Factors);

loadm path = ^resPathName Asset_Weights;
loadm path = ^resPathName Asset_RC;
loadm path = ^resPathName Factor_RC;
loadm path = ^resPathName FRC_Index;

Index_Asset = zeros(nDates,nIters);
Index_Strategy = zeros(nDates,nIters);
Begin_Date = zeros(nIters,1);
End_Date = zeros(nIters,1);

iter = 1;
do until iter > nIters;
  Weights = arraytomat(Asset_Weights[iter,.,.]);
  {Begin_Date[iter],End_Date[iter]} = indn_nomiss(Weights[.,1]);
  Weights = missrv(weights,index_weights);
  R = sumr(Weights .* R_SubIndex);
  R[1] = 0;
  Index_Asset[.,iter] = 100*cumprodc(1 +  R);
  iter = iter + 1;
endo;

indx_t = seqa(Begin_Date[1]-1,1,End_Date[1]-Begin_Date[1]+2);

let string ptitle = "\216Index" "\216Vol" "\216ES" "\216CF";
let string ptitle2 = "\216Vol" "\216ES" "\216CF";
let indx_method = 1 5 6 7;

pqgwin many;

graphset;
  begwind;
  window(2,2,0);
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;

  _pboxctl = {.25 0 3 10 90};
  _pcolor = seqa(1,1,9)|12;

  ytics(0,50,10,2);

  iter = 1;
  do until iter > 4;
    setwind(iter);     
      title(ptitle[iter]);
      y = arraytomat(Asset_Weights[indx_method[iter],indx_t,.]);
      call box(0,100*y);
    iter = iter + 1;
  endo;

  graphprt("-c=2 -cf=chap5-2-2-6a.eps");

  endwind;

graphset;
  begwind;
  window(2,2,0);
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;

  _pboxctl = {.25 0 3 10 90};
  _pcolor = seqa(1,1,9)|12;

  ytics(0,50,10,2);

  iter = 1;
  do until iter > 4;
    setwind(iter);     
      title(ptitle[iter]);
      y = arraytomat(Asset_RC[indx_method[iter],indx_t,.]);
      if iter == 4;
        y = y ./ sumr(y);
      endif;
      call box(0,100*y);
    iter = iter + 1;
  endo;

  graphprt("-c=2 -cf=chap5-2-2-6b.eps");

  endwind;


graphset;
  begwind;
  window(2,2,0);
  
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;
  
  _pboxctl = {.25 0 3 10 90};
  _pcolor = seqa(1,1,9)|12;

  ytics(0,100,25,2);
  let xlab = "F]1[" "F]2[" "F]3[" "F]4+[";
  asclabel(xlab,0);

  iter = 1;
  do until iter > 4;
    setwind(iter);     
      title(ptitle[iter]);
      y = arraytomat(Factor_RC[indx_method[iter],indx_t,.]);
      call box(0,100*y);
    iter = iter + 1;
  endo;

  graphprt("-c=2 -cf=chap5-2-2-6c.eps");

  endwind;

Index_Asset = 100*Index_Asset./Index_Asset[indx_t[1],.];

Index_Strategy = Index_Asset;
Index_Strategy_Smoothed = Index_Strategy;

iter = 1;
do until iter > nIters;
  c = csspline(t[indx_t],Index_Strategy[indx_t,iter],1,1);
  Index_Strategy_Smoothed[indx_t,iter] = fspline(c,t[indx_t]);
  iter = iter + 1;
endo;

graphset;
  window3u;
  
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10; _pcolor = 1|4; _pltype = 6|3; _pstype = 8|8; _plctrl = 0;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;
  _plegstr = "Asset-Weighted\000RFP-weighted";

  xtics(2006,2013,1,2);
  ytics(80,115,5,2);

  iter = 1;
  do until iter > 3;
    setwind(iter);     
      title(ptitle2[iter]);
      if iter == 1;
        _plegctl = {2 5 9 4};
      else;
        _plegctl = 0;
      endif;
      xy(t[indx_t],Index_Strategy_Smoothed[indx_t,indx_Method[1|(iter+1)]]);
    iter = iter + 1;
  endo;

  graphprt("-c=2 -cf=chap5-2-2-6d.eps");

  endwind;

data = dates[indx_t]~Index_Strategy[indx_t,.];
data[.,2:nIters+1] = 100*data[.,2:nIters+1]./data[1,2:nIters+1];

stats = zeros(9,nIters);

iter = 1;
do until iter > nIters;
  report = Reporting_Backtest(Data[.,1],Data[.,1+iter],Data[.,2],0,0,0);
  stats[1,iter] = vread(Report,"mu");
  stats[2,iter] = vread(Report,"sigma");
  stats[3,iter] = vread(Report,"MDD");
  stats[4,iter] = vread(Report,"sk");
  stats[5,iter] = vread(Report,"kt")-3.0;

  tmp = arraytomat(FRC_Index[iter,indx_t,.]);
  tmp = meanc(packr(tmp));
  stats[6:9,iter] = tmp[2 3 4 7];

  iter = iter + 1;
endo;

stats[1:3,.] = stats[1:3,.] * 100;
print ftosa(stats,"%lf",3,2);

