/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 272 (Figure 6.1)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.03 0.08;
let sigma = 0.05 0.15;
let rho =   1.00 
            0.20  1.00;
rho = xpnd(rho);
Cov = rho .* sigma .* sigma';

Bounds = -1~2;

gamma_ = seqa(-2,0.05,65);
{x,er,risk,gamma_,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,gamma_,0);

r = 0.01;
sh = (er-r)./risk;
indx = maxindc(sh);

er_p = r|mu|er[indx];
sigma_p = 0|sigma|risk[indx];

sh = sh[indx];
x_sh = 0.17;
y_sh = r + sh .* x_sh;

let x1 = 0.02 0.05 0.12;
y1 = r + sh .* x1;

let w = 0.10 0.40 0.80;
x2 = sqrt( (1-w)^2 .* sigma[1]^2 + w^2 .* sigma[2]^2 + 2 .* rho[1,2] .* w .* (1-w) .* sigma[1] .* sigma[2]);
y2 = (1-w) .* mu[1] + w .* mu[2];

let w1 = 0.20 0.50 0.50 0.30 0.00;
let w2 = 0.00 0.10 0.40 0.65 1.00;
w1 = rev(w1);
w2 = rev(w2);
w3 = 1-w1-w2;
x3 = sqrt( w1^2 .* sigma[1]^2 + w2^2 .* sigma[2]^2 + 2 .* rho[1,2] .* w1 .* w2 .* sigma[1] .* sigma[2]);
y3 = w1 .* mu[1] + w2 .* mu[2] + w3 .* r;

graphset;
  window3d;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected Return (in %)");
  xtics(0,25,5,2);
  ytics(0,12,2,2);

  setwind(1);
    title("\216Mean-variance Portfolios");
    _psym = 100*sigma_p~100*er_p~ones(4,1).*(8~5~12~1~10);
    _pmsgstr = "Cash\000Bond\000Equity\000Tangency portfolio";
    _pmsgctl = 100*sigma_p+0.75~100*er_p-0.35~ones(4,1).*(0.25~0~1~2~7) ;
    _pline = 1~3~0~100*r~100*x_sh~100*y_sh~1~4~10;
    xy(100*Risk,100*ER);

  setwind(2);
    title("\216Capital Market Line");
    _pmsgstr = "(C)\000(M)\000(A)";
    _pmsgctl = 100*x1-1.7~100*y1+0.5~ones(3,1).*(0.25~0~1~2~7) ;
    _psym = 100*x1~100*y1~ones(3,1).*(11~5~2~1~9) ;    
    xy(100*Risk,100*ER);

  setwind(3);
    title("\216Diversified Funds");
    _psym = 100*x2~100*y2~ones(3,1).*(11~5~2~1~9) ;    
    _pmsgstr = "(Def.)\000(Bal.)\000(Dyn.)";
    _pmsgctl = 100*x2+0.35~100*y2-0.750~ones(3,1).*(0.25~0~1~2~7) ;
    xy(100*Risk,100*ER);

    graphprt("-c=2 -cf=chap6-1-1-1.eps");

  endwind;


