/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 276 (Figure 6.5)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data\\";
datFileName = PathName $+ "msci-wgbi-djubs-fed"; 
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

Begin_Date = 19981231;
End_Date = 20111230;

data = selif(data,(data[.,1] .>= Begin_Date) .and (data[.,1] .<= End_Date));
data = Fill_Miss(data,4);

Dates = data[.,1];
nDates = rows(Dates);

Data = Data[.,2 3];
Data = 100 * Data ./ Data[1,.];

{cov,vol,cor} = Rolling_Covariance(Dates,Data,260,1);

let w1 = 0.20 0.80;
let w2 = 0.50 0.50;
let w3 = 0.80 0.20;

prc1 = zeros(nDates,2);
prc2 = zeros(nDates,2);
prc3 = zeros(nDates,2);

t = 1;
do until t > nDates;

  Sigma = arraytomat(cov[t,.,.]);

  {vol,mr,rc,prc} = Compute_Risk_Contribution(w1,Sigma);
  prc1[t,.] = prc';

  {vol,mr,rc,prc} = Compute_Risk_Contribution(w2,Sigma);
  prc2[t,.] = prc';

  {vol,mr,rc,prc} = Compute_Risk_Contribution(w3,Sigma);
  prc3[t,.] = prc';

  t = t + 1;
endo;

t = selif(seqa(1,1,nDates),Dates .>= 20000101);
prc1 = prc1[t,.];
prc2 = prc2[t,.];
prc3 = prc3[t,.];

b = seqa(-50,200/40,41);
Freqs = zeros(rows(b),2);

i = 1;
do until i > 2;
  {b,m,freqs[.,i]} = histp(100*prc1[.,i],b);
  i = i + 1;
endo;
freq1 = freqs ./ sumc(freqs)';
freq1 = miss(freq1,0);

i = 1;
do until i > 2;
  {b,m,freqs[.,i]} = histp(100*prc2[.,i],b);
  i = i + 1;
endo;
freq2 = freqs ./ sumc(freqs)';
freq2 = miss(freq2,0);

i = 1;
do until i > 2;
  {b,m,freqs[.,i]} = histp(100*prc3[.,i],b);
  i = i + 1;
endo;
freq3 = freqs ./ sumc(freqs)';
freq3 = miss(freq3,0);

graphset;
  window3u;

  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2;
  _pnumht = 0.20; _paxht = 0.20; _ptitlht = 0.25;

  _pbartyp = {6 1, 3 2}; _pbarwid = -1;

  xtics(-20,120,20,2);
  xlabel("\216Performance contribution (in %)");
  ylabel("\216Frequency (in %)");

  ytics(0,25,5,2);

  setwind(1);
    title("\216Defensive");
    call bar(m,100*freq1);

  setwind(2);
    title("\216Balanced");
    call bar(m,100*freq2);

  ytics(0,50,10,2);

  setwind(3);
    title("\216Dynamic");
    _plegstr = "Equity\000Bond";
    _plegctl = {2 6 5 9};
    call bar(m,100*freq3);

    graphprt("-c=2 -cf=chap6-1-2-2.eps");

  endwind;
